#include "osindependence.h"
/*#include "driver.h" */
#include "driverdef.h"
#include "pwrcryptioctl.h"
#include "km_interface.h"
#include "kmi_private.h"
#include "calcreqpp.h"  /* for CalcReqCheckAndPrepare() */

#include "globobj.h"

#define PWRCRYPT_DEVICE_STRING  "PowerCrypt (Hi/fn 7751 based) board"

BOOLEAN bCanWork = FALSE;   /* switched to TRUE on successful driver init. */
int     nMaxDevicesToAcquire = 10;

/****************************************************************************
****************************************************************************/

#define UNIT(x) minor(x)

/****************************************************************************
****************************************************************************/

/*
 * =STD=
 * Good old and loved by many driver function table
 * (cdevsw means "character device switch")
 */

#define CDEV_MAJOR 55

static d_open_t     PwrCryptOpen;
static d_close_t    PwrCryptClose;
static d_ioctl_t    PwrCryptIoctl;

static struct cdevsw PwrCrypt_cdevsw =
    {
	/* open */	PwrCryptOpen,
	/* close */	PwrCryptClose,
	/* read */	noread,
	/* write */	nowrite,
	/* ioctl */	PwrCryptIoctl,
	/* poll */	nopoll,
	/* mmap */	nommap,
	/* strategy */	nostrategy,
	/* name */	"PwrCrypt",
	/* maj */	CDEV_MAJOR,
	/* dump */	nodump,
	/* psize */	nopsize,
	/* flags */	0,
	/* bmaj */	-1
    };

static int PwrCryptPciProbe __P((device_t));
static int PwrCryptPciAttach __P((device_t));

static device_method_t pwc_methods[] = {
	/* Device interface */
	DEVMETHOD(device_probe, PwrCryptPciProbe),
	DEVMETHOD(device_attach, PwrCryptPciAttach),
	{ 0, 0 }
};

static driver_t pwc_driver = {
	"pwc",
	pwc_methods,
	1
};

static devclass_t pwc_devclass;

DRIVER_MODULE(pwc, pci, pwc_driver, pwc_devclass, 0, 0);

/****************************************************************************
****************************************************************************/

/*
 * =STD= for PCI devices
 * Declare and initialize "static struct   pci_device PwrCryptPCIDevice".
 * This PwrCryptPCIDevice contains ptrs to Probe and Attach funcs.
 * Then DATA_SET macro adds the ptr to this struct to the misterious
 * "pcidevice_set".
 * This is probably how the system loader gets to those funcs.
 *
 * I suppose that these pci_device structs are gathered in a list at the kernel
 * build time. When the kernel is loading and comes to its "PCI device
 * configuration phase", it uses that list to find all Probe and Attach funcs
 * drivers have added to the list.
 *
 * Looks like ISA devices too have such a thing, only called something like
 * "isa_device" and probably added to another table and handled in the similar
 * manner.
 */

/****************************************************************************
=STD= for PCI devices
    Called by system to ask the driver whether it wants to work with a device
    with the given PCI device ID. Thus this function is essentially a filter.

    It merely checks it against its device ID and returns an address of a text
    string, or NULL if it is not its device ID. DevID has the vendor ID in the
    lower word and the devive ID in the higher word

    It shouldn't touch the device at this time.

    DevID - ULONG containing VendorID/DeviceID
    CfgID - a cookie assembled from the bus#, device# and function#

****************************************************************************/
int probe_cnt=0;
int good_probe_cnt=0;

static int
PwrCryptPciProbe(dev)
	device_t	dev;
{
	u_int32_t		DevID;
    char    *pszRetStr;

	DevID = pci_get_vendor(dev) | pci_get_device(dev) << 16;
    DPF("PwrCryptProbe DevID=%08x\n", DevID);
    probe_cnt++;
    pszRetStr = NULL;
    if ( DevID == (PWRCRYPT_PCI_DEVICE_ID << 16) + PWRCRYPT_PCI_VENDOR_ID)
    {
        DPF("   DevID match!\n");

        good_probe_cnt++;

        /************************************************************************
         * Acquire only the number of devices specified in the configuration
         */
        if ( good_probe_cnt <= nMaxDevicesToAcquire &&
             good_probe_cnt <= MAXIMUM_DEVICES_SUPPORTED)
        {
            DPF("    Acquiring this device\n");
            pszRetStr = PWRCRYPT_DEVICE_STRING;
        }
        else
        {
            printf("    Refusing the device on successful probe #%d:\n"
                   "    device limit imposed by %s exceeded\n",
                 good_probe_cnt,
                 ( good_probe_cnt > MAXIMUM_DEVICES_SUPPORTED) ?
                    "constant MAXIMUM_DEVICES_SUPPORTED" :
                    "driver configuration"
                );
        }
    }
/*    DPF("PwrCryptProbe #%d/%d: DevID=%08x\n", good_probe_cnt, probe_cnt, DevID); */
	if (pszRetStr == NULL)
		return(ENXIO);
	device_set_desc(dev, pszRetStr);
	return(0);
}
/****************************************************************************
****************************************************************************/

/****************************************************************************
=STD= for PCI devices
    Called by system to ask the driver to initialize its device.
    If multiple devices are installed, called multiple times and "unit"
    selects the device instance.

    Notice that the func has no return value, so when the system calls it,
    this is not the question whether the driver will agree to service
    the device. That has already been decided, based, probably, on the call
    to the Probe().

****************************************************************************/
static
int
PwrCryptPciAttach(dev)
	device_t		dev;
{
    u_long old_irq;
    u_long new_irq, status;
    PPROCBASE pNewProcBase;

    DPF("PwrCryptAttach: dev=%08x, unit=%x\n", dev, device_get_unit(dev));

    PwrCryptDriverInit();

    /************************************************************************
     * Initialize the driver's permanent/global data.
     */
    GlobalObjectsConstructor( &glob);

    status = GlobalObjectsInit( &glob);
    if ( !BSD_SUCCESS( status))
    {
        DPF( "Unable to initialize global objects. status=0x%08x\n", status);
        GlobalObjectsDestructor( &glob);
        return status;
    }

    /************************************************************************
     * Create 1 PROC_BASE object for each device.
     *
     * Notice - configuration limitation on the number of devices is placed by
     * the Probe(). Here we only check if we are going to run over the array
     * bounds which shouldn't happen if each successful Probe() induces no more
     * than one Attach().
     */
    if ( glob.NumOfValidProcBases == MAXIMUM_DEVICES_SUPPORTED)
    {
        printf("PwrCryptAttach: failed. Unit #%d: device limit (%d devices) exceeded\n", device_get_unit(dev), MAXIMUM_DEVICES_SUPPORTED);
        return(ENXIO);
    }

    ASSERT( glob.pProcBase[ glob.NumOfValidProcBases] == NULL);
    DPF( "    allocating 0x%04x bytes for PROCBASE[%d]\n", sizeof( PROCBASE), glob.NumOfValidProcBases);

    pNewProcBase = malloc( sizeof( *pNewProcBase), M_DEVBUF, M_WAITOK);
    ProcBaseConstructor( pNewProcBase, &glob.Config);

    if ( ProcBaseAcquireHardware( pNewProcBase,
                                  &glob.Dispatcher,
                                  dev))
    {
        glob.pProcBase[ glob.NumOfValidProcBases] = pNewProcBase;
        glob.NumOfValidProcBases ++;
    }
    else
    {
        if ( !ProcBaseReleaseHardware( pNewProcBase))
        {
            printf("Release HW failed\n");
        }
        ProcBaseDestructor( pNewProcBase);
        free( pNewProcBase, M_DEVBUF);
    }

    pwrcrypt_load();

	return(0);
}

/****************************************************************************
=STD=
    Device open handler
****************************************************************************/
static int open_cnt = 0;

static
int
PwrCryptOpen(
    dev_t dev,
    int flags,
    int fmt,
    struct proc *p
    )
{
    if ( !bCanWork)
    {
        return ENXIO;
    }

    open_cnt++;
    DPF("PwrCryptOpen #%d. pProc %08x\n", open_cnt, p);
    return 0;
}

/****************************************************************************
=STD=
    Device close handler
****************************************************************************/

/*---------------------------------------------------------------------------
PCFreeSlotCallback()

This function is called by the SlotTrackerEnumSlotsByKey() for each slot
which associated "key" matches the "key" passed to the routine.
This function frees the specifieed slot.
---------------------------------------------------------------------------*/
static
void
PCFreeSlotCallback(
    IN PVOID    Context,
    IN ULONG    SlotNumber
    )
{
    struct proc *pProc;

    BOOLEAN     bFreeSuccess;
    IOCTL_PWRCRYPT_FREE_SLOT_INPUT_OUTPUT FreeSlotPack;
    BSDSTATUS   status;

    DPF("Free slot %08x\n", SlotNumber);

    pProc = (struct proc *) Context;

    /* Tell the SlotTracker to verify the passed slot number against
     * the key (struct proc ptr) and free its remembered association
     * between the reserved slot number and the struct proc for which
     * the IRP has been issued.
     *
     * Note! In this callback, the slot number will always be correct,
     * since it is given by SlotTracker itself. And it will always
     * be associated with the Context, since we use it as the key,
     * when asking the SlotTracker to enumerate slots by key.
     */
    bFreeSuccess = SlotTrackerUnregisterSlot(
                                    &glob.SlotTracker,
                                    SlotNumber,
                                    (ULONG) pProc,
                                    &FreeSlotPack.nSlotNumberToFree);
    
    DPF("SlotTracker freed slot %ld\n", FreeSlotPack.nSlotNumberToFree);
    /*
     * FreeSlot failure can only happen if the slot's RefCnt is not zero,
     * but this must not be the case here, because we are called from
     * IRP_MJ_CLOSE handler when there can't be outstanding IRPs for the
     * corresponding file object and hence its associated slots.
     */
    ASSERT( bFreeSuccess);

    /*
     * After SlotTrackerFreeSlot, nobody has access to the slot until it is
     * reserved again, but before that can happen, the slot must be freed via
     * the following call to the PwrCrypt service.
     */
    status = PwrcryptFreeSlot( &FreeSlotPack);

    ASSERT( BSD_SUCCESS( status));
}

/*---------------------------------------------------------------------------
The close() itself
---------------------------------------------------------------------------*/
static int close_cnt = 0;

static
int
PwrCryptClose(
    dev_t dev,
    int flags,
    int fmt,
    struct proc *p
    )
{
    if ( !bCanWork)
    {
        return ENXIO;
    }

    close_cnt++;
    DPF("PwrCryptClose #%d. pProc %08x\n", close_cnt, p);

    /*
     * Free all slots associated with the given file object.
     * Note the trick below - we call the SlotTracket to enumerate slots by
     * key (file object ptr) and give it its own slot freeing method as
     * callback.
     */
    SlotTrackerEnumSlotsByKey(
            &glob.SlotTracker,
            PCFreeSlotCallback,     /* callback address */
            (PVOID) p,              /* Context */
            (ULONG) p);             /* Key */

    return 0;
}

/****************************************************************************

CalculateCompletionRoutine()

Description:
    Called by the core PwrCrypt driver when it completes the execution of the
    calc.request in PwrcryptCalculate().

Arguments:

    nComplRtnParm1 - ptr to the event to be signaled
    nComplRtnParm2 - not used

****************************************************************************/
static
void
CalculateCompletionRoutine
    (
        ULONG   nComplRtnParm1,
        ULONG   nComplRtnParm2
    )
{
    DPF( "CalculateCompletionRoutine - signalling the event\n");
    ASSERT( nComplRtnParm2 == 0x12345678);
    PCrptSignalEvent( (PPCRPT_EVENT) nComplRtnParm1);
}

/****************************************************************************
=STD=
    Device IOCTL handler.

    Remember, that all our IOCTL codes are defined via IOWR(), so the system
    makes a work copy of the user request in non-paged memory, then calls us
    and gives ptr to it in arg, then, when we return, copies the work copy
    back over the original user request and returns to the user

****************************************************************************/

/*
 * CALC_REQ_CONTAINER
 *
 * It's funny, but the copy of the IOCTL packet (*arg) that the
 * system makes is actually placed on the process's kernel mode stack. The
 * DPF() below can demonstrate this - values of "arg" and the address of the
 * local var "status" are very close.
 *
 * When I learnt about that, I had understood why the size of the IOCTL
 * packet is limited to, if I remember it right, 4KB. The process's kernel
 * mode stack is used for handling page faults. Then it must be non-pageable
 * and, hence, small.
 *
 * Since the kernel mode stack is process-specific and is mapped into the
 * process's private address space, it can't be accessed in arbitrary process
 * context. Since we are indeed planning to touch the IOCTL packet at
 * interrupt time, we unfortunately have to make another copy of it in the
 * globally accessed KM memory obtained from the standard allocator.
 *
 * More in 4.4BSD p.86
 *
 * Also, an event is involved in stopping the current process until the
 * calc.request is fulfilled. The event is signalled at interrupt time too,
 * so it goes to the "container" together with the copy of the calc.request.
 */
typedef struct _CALC_REQ_CONTAINER
{
    IOCTL_PWRCRYPT_CALCULATE_INPUT_OUTPUT   CalcReq;
    PCRPT_EVENT                             CompletionEvent;
}
CALC_REQ_CONTAINER, *PCALC_REQ_CONTAINER;

static
int
PwrCryptIoctl
    (
        dev_t       dev,
        u_long         CtlCode,
        caddr_t     arg,
        int         flag,
        struct proc *pProc
    )
{
    int         status;  

        int nSleepResult;

#ifdef notdef
    int         unit;
    unit = UNIT(minor(dev));
    if (unit >= MAX_NUMBER_OF_CHIPS)
    {
        /* unit out of range */
        return ENXIO; */
    }
#endif
 
    if ( !bCanWork)
    {
        return ENXIO;
    }

    if ( arg == NULL)
    {
        return EINVAL;
    }

    status = STATUS_SUCCESS;

    DPF( "PwrCryptIoctl: CtlCode=%08x Arg=%08x &status=%08x\n", CtlCode, arg, &status);

    switch (CtlCode)
    {
        case IOCTL_PWRCRYPT_CALCULATE:
        {
            PIOCTL_PWRCRYPT_CALCULATE_INPUT_OUTPUT  pCalcReqOrig;
            PCALC_REQ_CONTAINER                     pCalcReqContainer;

            PIOCTL_PWRCRYPT_CALCULATE_INPUT_OUTPUT  pCalcReq;
            PPWRCRYPT_CALCULATE_KM_WORKAREA         pWA;

            pCalcReqOrig = ( PIOCTL_PWRCRYPT_CALCULATE_INPUT_OUTPUT) arg;

            DPF( "    making container %08x(%08x:%08x)",
                 sizeof(*pCalcReqContainer),
                 sizeof(*pCalcReq),
                 sizeof(pCalcReqContainer->CompletionEvent)
               );

/*            nSleepResult = tsleep( pCalcReqOrig, PRIBIO | PCATCH, "IoEvWt", 5*hz); */

            pCalcReqContainer =
                ( PCALC_REQ_CONTAINER)
                malloc( sizeof(*pCalcReqContainer), M_TEMP, M_WAITOK);

            pCalcReq = &pCalcReqContainer->CalcReq;

            memcpy( pCalcReq, pCalcReqOrig, sizeof(*pCalcReq));

            /*
             * Setup the pointer to the KM work area inside of the new copy
             * of the calc.req in the "container". Note that this routine
             * touches only KMWorkArea in the CalcReq, other members are filled
             * by the calling app.
             */
            pWA = (PPWRCRYPT_CALCULATE_KM_WORKAREA) &pCalcReq->KMWorkArea;

            /*
             * Check Command-related members of
             * the CalcReq and fill Command-related members of CalcReq.KMWorkArea
             */
            if ( CalcReqCheckAndPrepare( pCalcReq, sizeof( *pCalcReq)))
            {
                if ( SlotTrackerReferenceSlot( &glob.SlotTracker,
                                               pCalcReq->nSlotNumber,
                                               (unsigned long) pProc))
                {
                    /*
                     * The IOCTL input buffer contains ptrs to 2 data buffers.
                     * These ptrs point to the application's private address space.
                     * Indicate this fact by setting appropriate flags, so that the
                     * driver would know how to handle them (it will attempt to
                     * locks them and build MDLs).
                     *
                     * Also, since our caller is a user-mode app, the buffers
                     * must be verified on access from the user mode
                     */
                    pWA->flBufsDescType = ( PWRCRYPT_CALCULATE_IN_PAGE |
                                            PWRCRYPT_CALCULATE_IN_USER |
                                            PWRCRYPT_CALCULATE_OUT_PAGE |
                                            PWRCRYPT_CALCULATE_OUT_USER
                                          );

                    /*
                     * It's known that the calc.service is called in the requesting
                     * process context, so we set this var to TRUE, so that the
                     * process's quota would be charged when the PwrCrypt driver
                     * will create MDLs.
                     */
                    pWA->bChargeQuota = TRUE;

                    /* Report the completion routine address and its two parms
                     * to the service. It doesn't interpret the parms, just
                     * calls the routine and passes them to it, upon request
                     * completion. We need to see the CompletionEvent in our
                     * completion routine and set this event.
                     */
                    pWA->pComplRtn = CalculateCompletionRoutine;
                    pWA->nComplRtnParm1 = (ULONG) &pCalcReqContainer->CompletionEvent;
                    pWA->nComplRtnParm2 = 0x12345678;

                    /* Initialize event _before_ calling the calculating
                     * service - the service will call our callback routine
                     * at any time and the routine will signal the event.
                     */
                    PCrptInitEvent( &pCalcReqContainer->CompletionEvent);

                    /*
                     * Call the PwrCrypt calculating service
                     */
                    status = PwrcryptCalculate( pCalcReq);

                    /* By not returning SUCCESS, the service refuses to process the
                     * request. This may be caused by insufficient resources or bad
                     * user data buffers.
                     *
                     * It it DOES return SUCCESS, it will DO call the completion
                     * routine and we must not touch the IRP anymore, since the
                     * completion routine _is_ to complete this IRP and, in fact,
                     * it is possible that it has been already completed by the
                     * time when the service returns. The status is checked at
                     * the bottom of this IOCTL dispatcher routine to decide
                     * whether to complete the IRP.
                     */
                    if ( status == STATUS_SUCCESS)
                    {
                        /*
                         * The service has accepted the calc.request -
                         * yield the CPU and wait for the event signaling
                         */
                        PCrptWaitOnEvent( &pCalcReqContainer->CompletionEvent);

                        /*
                         * The event has been signalled - the calc.request has
                         * been fulfilled. Copy the two OUT fields from the copy
                         * of the IOCTL packet inside of our "container" to the
                         * original system copy on the stack.
                         *
                         * Attention - this copy operation's calculations are
                         * based on the fact that the nResultSize and Result
                         * are the first fields of the CalcReq.
                         * We need to copy back to user the nResultSize + the
                         * effective part of the Result field, which size is
                         * indicated by the nResultSize.
                         */
                        ASSERT( FIELD_OFFSET(
                                    IOCTL_PWRCRYPT_CALCULATE_INPUT_OUTPUT,
                                    nResultSize) == 0);

                        /* */
                        /* BUGBUG - delete this */
                        /* */
/*                        pCalcReq->nResultSize = 0; */

                        memcpy(
                            pCalcReqOrig,
                            pCalcReq,
                            FIELD_OFFSET(
                                IOCTL_PWRCRYPT_CALCULATE_INPUT_OUTPUT,
                                Result) +
                            pCalcReq->nResultSize);
                    }

                    if ( !SlotTrackerDereferenceSlot(
                                &glob.SlotTracker,
                                pCalcReq->nSlotNumber,
                                (unsigned long) pProc))
                    {
                        /* BUGBUG: unexpected condition - slot remains "busy" */
                        printf("Unexpected error - slot dereferencing has failed. slot 0x%04x\n", pCalcReq->nSlotNumber);
                    }
                } /* if ReferenceSlot */
                else
                {
                    printf("Invalid slot number\n");
                    status = EINVAL;
                }
            } /* if IOCTL parms are valid */
            else
            {
                printf( "Invalid calculating request\n");
                status = EINVAL;
            }

            free( pCalcReqContainer, M_TEMP);

            break;
        }

        case IOCTL_PWRCRYPT_RESERVE_SLOT:
        {
            PIOCTL_PWRCRYPT_RESERVE_SLOT_INPUT_OUTPUT   pResSlotReq;

            pResSlotReq = (PIOCTL_PWRCRYPT_RESERVE_SLOT_INPUT_OUTPUT) arg;

            DPF( "RESERVE\n");

            /* 
             * Call the PwrCrypt slot reservation service first.
             */
            status = PwrcryptReserveSlot( pResSlotReq);
/*            ASSERT( status != STATUS_PENDING); */
            if ( BSD_SUCCESS( status))
            {
                BOOLEAN bRegSuccess;

                /*
                 * Tell the SlotTracker to remember the association between
                 * the reserved slot number and the file object for which
                 * the IRP has been issued.
                 * Note that we also substitute a Slot Tracker index
                 * in the place of the PwrCrypt core slot number in the
                 * request packet, so the user mode caller receives
                 * Slot Tracker's index instead of PwrCrypt's slot number.
                 */
                bRegSuccess = SlotTrackerRegisterSlot(
                                    &glob.SlotTracker,
                                    pResSlotReq->nReservedSlotNumber,
                                    (ULONG) pProc,
                                    &pResSlotReq->nReservedSlotNumber);

                ASSERT( bRegSuccess);
            }
            break;
        }

        case IOCTL_PWRCRYPT_FREE_SLOT:
        {
            PIOCTL_PWRCRYPT_FREE_SLOT_INPUT_OUTPUT    pFreeSlotReq;

            pFreeSlotReq = (PIOCTL_PWRCRYPT_FREE_SLOT_INPUT_OUTPUT) arg;

            DPF( "FREE\n");

            /* Tell the SlotTracker to verify the passed slot number against
             * the key (file obj ptr) and free its remembered association
             * between the reserved slot number and the key (file object ptr
             * for which the IRP has been issued).
             * Note that we also substitute the PwrCrypt core slot number
             * in the place of the Slot Tracker index in the request packet, so the user mode caller receives
             * before calling to the PwrCrypt with that packet.
             */
            if ( SlotTrackerUnregisterSlot(
                                    &glob.SlotTracker,
                                    pFreeSlotReq->nSlotNumberToFree,
                                    (ULONG) pProc,
                                    &pFreeSlotReq->nSlotNumberToFree))
            {
                /*
                 * SlotTracker has said "Ok" - call the PwrCrypt service.
                 */
                status = PwrcryptFreeSlot( pFreeSlotReq);
/*                ASSERT( status != STATUS_PENDING); */
                if ( !BSD_SUCCESS( status))
                {
                    DPF1("Slot freeing failed\n");
                }
            }
            else
            {
                DPF1("Attempt to free a wrong slot\n");
                status = EINVAL;
            }
            break;
        }

        case IOCTL_PWRCRYPT_GET_CAPS:
        {
            PIOCTL_PWRCRYPT_GET_CAPS_INPUT_OUTPUT    pGetCapsReq;

            pGetCapsReq = (PIOCTL_PWRCRYPT_GET_CAPS_INPUT_OUTPUT) arg;

            DPF( "GETCAPS\n");

            /*
             * Call the PwrCrypt service.
             */
            status = PwrcryptGetCaps( pGetCapsReq);
/*            ASSERT( status != STATUS_PENDING); */
            break;
        }

        default:
        {
            DPF( "INVALID\n");
            status = ENOTTY;
            break;
        }
    }
    return status;
}

int     load_cnt=0;

static
int
pwrcrypt_load ()
{
    BSDSTATUS   status;

    /* The "get capabilities" request sent to PwrCrypt
     */
    IOCTL_PWRCRYPT_GET_CAPS_INPUT_OUTPUT    GetCapsPack;


    load_cnt++;
    DPF( "    PwrCrypt_load %d\n", load_cnt);

    /*
     * Now, the glob.Config is alive and can be used
     */

    /*
     * This may be useful for configuration flexibility.
     */
    nMaxDevicesToAcquire = ConfigGetDeviceCntLimit( &glob.Config);

#ifndef _NO_HW_
    if (0)
    {
        printf("    pci_register_lkm failed\n");
        GlobalObjectsDestructor( &glob);
        return EACCES;
    }
    else
    {
        DPF("    pci_register_lkm succeeded\n");
        if ( glob.NumOfValidProcBases != 0)
        {
            DPF("    %d devices found, enabling KM interface\n", glob.NumOfValidProcBases);
            KmInterfaceEnable();
        }
        else
        {
            printf("    no devices initialized\n");
            GlobalObjectsDestructor( &glob);
            return ENXIO;
        }
    }
#else
    KmInterfaceEnable();
#endif

    /************************************************************************
     * UM interface part initialization.
     * We don't destruct the core driver's global objects even on error -
     * they are not ours to destruct
     */
    {
        ULONG   SlotsToTrack;

        /************************************************************************
         * Now that we know we have proper access to PwrCrypt driver, call its
         * service to retrieve the maximum number of slots it exposes to clients.
         * Though we check the return status, the service should never normally
         * fail. Then, using the number of slots obtained from PwrCrypt,
         * initialize SlotTracker
         */
        #ifndef _NO_HW_
            status = PwrcryptGetCaps( &GetCapsPack);
            if ( !BSD_SUCCESS( status))
            {
                printf( "Can't get PwrCrypt capabilities. Status = %x \n", status);
                GetCapsPack.nNumOfDESSlots = 0;
                GetCapsPack.nNumOfRC4Slots = 0;
                GetCapsPack.nNumOfCompHistSlots = 0;
            }
        #else
            GetCapsPack.nNumOfDESSlots = 0;
            GetCapsPack.nNumOfRC4Slots = 16;
            GetCapsPack.nNumOfCompHistSlots = 0;
        #endif

        SlotsToTrack = GetCapsPack.nNumOfDESSlots +
                       GetCapsPack.nNumOfRC4Slots +
                       GetCapsPack.nNumOfCompHistSlots;

        ASSERT( SlotsToTrack != 0);

        if ( SlotsToTrack != 0)
        {
            SlotTrackerInit( &glob.SlotTracker, SlotsToTrack);
            if ( SlotTrackerIsValid( &glob.SlotTracker))
            {
                /*
                 * "Unlock" the driver for user-mode requests
                 */
                bCanWork = TRUE;
            }
            else
            {
                DPF1("Can't initialize SlotTracker.\n");
            }
        }
    }

    if ( !bCanWork)
    {
        GlobalObjectsDestructor( &glob);
        return EACCES;
    }

    return STATUS_SUCCESS;
}

/****************************************************************************
=STD=
    Driver initialization routine, called by the OS BEFORE probing the
    devices - no device info is available at this time. 
    
****************************************************************************/

static boolean_t bCdevswAdded = FALSE;

static
void
PwrCryptDriverInit
    (
        void *unused
    )
{
    DPF("PwrCryptDriverInit\n");

    if ( !bCdevswAdded)
    {
	make_dev(&PwrCrypt_cdevsw, 0, 0, 0, 0666, "PwrCrypt");

        bCdevswAdded = TRUE;

        DPF("    cdevsw added\n");
    }
    else
    {
        DPF("    duplicate call detected\n");
    }
}
