/**********************************************************************

    Module: globobj.h

        Product: kernel mode driver for "PowerCrypt" encryption
        acceleration board based on Hi/fn 7751 chip
        by Ashley Laurent, Inc.

    ---------------------------------------------------------------

    Copyright:

        Ashley Laurent, Inc., 1999
        All Rights Reserved.

    ---------------------------------------------------------------

    Functional Description:

        Defines the structure of the driver's static global data area
        containing single-instanced global objects. They are initialized at
        the driver start

    ---------------------------------------------------------------

    Environment:

        NT and BSD kernel mode

    ---------------------------------------------------------------

    Author:

        Stan Shkolnyy

    ---------------------------------------------------------------

    Revision history:

        *   05-Mar-99   initial revision

**********************************************************************/

#ifndef __GLOBOBJ_H__
#define __GLOBOBJ_H__

#include "processor.h"
#include "dispatcher.h"
#include "slotman.h"
#include "config.h"
#include "slotrack.h"
#include "procbase.h"
/*#include "watch.h" */

/****************************************************************************
 * Driver global data structure
 */
typedef struct _GLOBAL_OBJECTS
{
    ULONG               NumOfValidProcBases;

/*    WATCH               Watch; */

    /* Command (client request) queue
     */
    DISPATCHER          Dispatcher;

    /* Slot manager
     */
    SLOTMAN             SlotMan;
    SLOTTRACKER         SlotTracker;

    /* Registry path to ProcessorX subkey under service control key
     */
/*    UNICODE_STRING      usProcRegPathCopyX; */

    /*
     * Array of ptrs to Processor bases which give birth to PROCESSORs
     */
    PPROCBASE           pProcBase[ MAXIMUM_DEVICES_SUPPORTED];

    /* Driver and processor configuration info withdrawn from the Registry
     */
    CONFIG              Config;

    /* Link device used by clients to declare their usage of the driver to
     * prevent it from unloading
     */
/*    PDEVICE_OBJECT      pLnkDev; */

} GLOBAL_OBJECTS, *PGLOBAL_OBJECTS;

extern  GLOBAL_OBJECTS  glob;

void
GlobalObjectsConstructor
    (
        PGLOBAL_OBJECTS pThis
    );

BSDSTATUS
GlobalObjectsInit
    (
        PGLOBAL_OBJECTS pThis
    );

void
GlobalObjectsDestructor
    (
        PGLOBAL_OBJECTS pThis
    );

#endif
