/**********************************************************************

    Module: briefcomm.h

        Product: kernel mode driver for "PowerCrypt" encryption
        acceleration board based on Hi/fn 7751 chip
        by Ashley Laurent, Inc.

    ---------------------------------------------------------------

    Copyright:

        Ashley Laurent, Inc., 1999
        All Rights Reserved.

    ---------------------------------------------------------------

    Functional Description:

        Defines the "brief" representation of the natural 7751 instruction
        and routines which transfer information (bits) between these two
        kinds of representation.

    ---------------------------------------------------------------

    Environment:

        NT or BSD kernel mode

    ---------------------------------------------------------------

    Author:

        Stan Shkolnyy

    ---------------------------------------------------------------

    Revision history:

        *   19-Mar-99   initial revision

**********************************************************************/

#ifndef __BRIEFCOMM_H__
#define __BRIEFCOMM_H__

#ifdef _WIN32
    #pragma pack(push, _BRIEFCOMM_STRUCT_DEF_, 1)
#else
/*    #warning No pragma pack() ??? */
#endif

/*
 * This member of the PWRCRYPT_CALCULATE_KM_WORKAREA structure is for PwrCrypt
 * internal use. The KM interface user doesn't touch it.
 */
typedef struct _BRIEF_COMMAND
{
    unsigned char   flags;
    unsigned char   newflags;
    unsigned short  SlotNumber;

} BRIEF_COMMAND, *PBRIEF_COMMAND;

/*
 * flags that can be set in the BRIEF_COMMAND's members "flags" and "newflags"
 */
#define MAC_ENGAGED         0x01
#define MAC_NEW_CTX         0x02    /* MACCommand.NewKey */
#define COMP_ENGAGED        0x04
#define COMP_NEW_CTX        0x08    /* CompCommand.ClearHistory */
#define CRYP_ENGAGED        0x10
#define CRYP_NEW_CTX        0x20    /* CrypCommand.NewKey */
#define CRYP_DONT_STORE_CTX 0x40    /* CrypCommand.DontStoreCtx */

#ifdef _WIN32
    #pragma pack(pop, _BRIEFCOMM_STRUCT_DEF_)
#endif

#endif
