/*++

Copyright (c) 1997-1998  Microsoft Corporation

Module Name:

    debug.c

Abstract:

    General debug print routine.  

Author:

    Steve Dziok (SteveDz)

Environment:

    Kernel mode

Revision History:

--*/

#include <bsdddk.h>

#if __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#include "debug.h"
#include "driverdef.h"

#if DEBUG
	#define DEBUG_BUFFER_SIZE  80 
	ULONG DebugPrintLevel = 3;
	UCHAR DebugBuffer[DEBUG_BUFFER_SIZE];
#endif /* DEBUG */

VOID
#if __STDC__
DebugPrintRoutine
    (
        ULONG PrintLevel,
        char const *DebugMessage,
        ...
    )
#else
DebugPrintRoutine
    (
        PrintLevel,
        DebugMessage,
        va_alist
    )
    ULONG PrintLevel;
    char *DebugMessage;
    va_dcl
#endif

/*++

Routine Description:

    Debug print routine.

Arguments:

    Debug print level between 0 and 3, with 3 being the most verbose.

Return Value:

    None.

--*/

/*
 * The func has body only in debug builds
 */
{
#if DEBUG
    va_list ap;

    #if __STDC__
        va_start(ap, DebugMessage);
    #else
        va_start(ap);
    #endif

        /*
         * DRIVER_DEBUG_NAME is expected to be defined somewhere in driverdef.h
         */
        printk(DRIVER_DEBUG_NAME ":");
        vsprintf( DebugBuffer, DebugMessage, ap);
	printk("%s", DebugBuffer);

    va_end(ap);

#endif /* DEBUG */
}
