/*

Copyright (c) 1990-1994  Microsoft Corporation

Module Name:

    bsdddk.h

Abstract:

    This module defines the BSD types, constants, and functions that are
    exposed to the PwrCrypt device driver.

Revision History:

GTGI	7-Oct-1999	Began preliminary modifications for the Linux port.
			
*/

#ifndef _BSDDDK_
#define _BSDDDK_

#define _LOOSE_KERNEL_NAMES

#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/ioctl.h>
#include <asm/processor.h>
#include <asm/bitops.h>
#include <asm/io.h>

#include "bsddef.h"
#include "bsdstatus.h"

/*
 * for DELAY()
 */
#include <linux/delay.h>

#ifndef DELAY /* XXX */
#define DELAY udelay
#endif /* !DELAY */



/*
 * If debugging support enabled, define an ASSERT macro that works.  Otherwise
 * define the ASSERT macro to expand to an empty expression.
 */

#ifdef DEBUG
#   define DPF printk
#   define DPF1 printk
#   ifndef ASSERT
#       define ASSERT(x) if (!(x)) { printk("** ASSERT FAILURE(" #x ") in %s(%d)\n",__FILE__,__LINE__); }
#   endif
#endif

#ifdef DEBUG1
#   define DPF1 printk
#   ifndef ASSERT
#       define ASSERT(x) if (!(x)) { printk("** ASSERT FAILURE(" #x ") in %s(%d)\n",__FILE__,__LINE__); }
#   endif
#endif

/*
 * BUGBUG
 */
#ifndef ASSERT
    #ifdef _LEAVE_ASSERTS_
        #define ASSERT(x) if (!(x)) { printk("** ASSERT FAILURE(" #x ") in %s(%d)\n",__FILE__,__LINE__); }
    #else
        #define ASSERT(x)
    #endif
#endif

#ifndef DPF
#   define DPF
#endif

#ifndef DPF1
#   define DPF1
#endif

/***************************************************************************
 */
/* Define ourselves a unique PCI tag for DevID. We want to do this because it
   appears that the NT-ish code in other parts of the tree will break if they
   aren't given an "atomic" CfgID variable, and linux doesn't have that out
   of the box. */

typedef struct {
	unsigned char bus;
	unsigned char function;
} linux_pcitag;

#define PCI_TYPE0_ADDRESSES	6

/***************************************************************************
 * Processor modes.
 */

typedef CCHAR KPROCESSOR_MODE;

typedef enum _MODE {
    KernelMode,
    UserMode,
    MaximumMode
} MODE;

/***************************************************************************
 * Define a Memory Descriptor List (MDL)
 *
 * An MDL describes pages in a virtual buffer in terms of physical pages.  The
 * pages associated with the buffer are described in an array that is allocated
 * just after the MDL header structure itself.  In a future compiler this will
 * be placed at:
 *
 *      ULONG Pages[];
 *
 * Until this declaration is permitted, however, one simply calculates the
 * base of the array by adding one to the base MDL pointer:
 *
 *      Pages = (PULONG) (Mdl + 1);
 *
 * Notice that while in the context of the subject thread, the base virtual
 * address of a buffer mapped by an MDL may be referenced using the following:
 *
 *      Mdl->StartVa | Mdl->ByteOffset
 */

typedef struct _MDL {
/*    struct _MDL *Next;         */
    CSHORT Size;
    CSHORT MdlFlags;
/*    struct _EPROCESS *Process; */
/*    PVOID MappedSystemVa;      */
    PVOID StartVa;
    ULONG ByteCount;
    ULONG ByteOffset;
} MDL, *PMDL;

#define MDL_MAPPED_TO_SYSTEM_VA     0x0001
#define MDL_PAGES_LOCKED            0x0002
#define MDL_SOURCE_IS_NONPAGED_POOL 0x0004
#define MDL_ALLOCATED_FIXED_SIZE    0x0008
#define MDL_PARTIAL                 0x0010
#define MDL_PARTIAL_HAS_BEEN_MAPPED 0x0020
#define MDL_IO_PAGE_READ            0x0040
#define MDL_WRITE_OPERATION         0x0080
#define MDL_PARENT_MAPPED_SYSTEM_VA 0x0100
#define MDL_LOCK_HELD               0x0200
#define MDL_SYSTEM_VA               0x0400
#define MDL_IO_SPACE                0x0800
#define MDL_NETWORK_HEADER          0x1000
#define MDL_MAPPING_CAN_FAIL        0x2000
#define MDL_ALLOCATED_MUST_SUCCEED  0x4000


#define MDL_MAPPING_FLAGS (MDL_MAPPED_TO_SYSTEM_VA     | \
                           MDL_PAGES_LOCKED            | \
                           MDL_SOURCE_IS_NONPAGED_POOL | \
                           MDL_PARTIAL_HAS_BEEN_MAPPED | \
                           MDL_PARENT_MAPPED_SYSTEM_VA | \
                           MDL_LOCK_HELD               | \
                           MDL_SYSTEM_VA               | \
                           MDL_IO_SPACE )

/*
 *
 * ULONG
 * ROUND_TO_PAGES (
 *     IN ULONG Size
 *     )
 *
 * Routine Description:
 *
 *     The ROUND_TO_PAGES macro takes a size in bytes and rounds it up to a
 *     multiple of the page size.
 *
 *     NOTE: This macro fails for values 0xFFFFFFFF - (PAGE_SIZE - 1).
 *
 * Arguments:
 *
 *     Size - Size in bytes to round up to a page multiple.
 *
 * Return Value:
 *
 *     Returns the size rounded up to a multiple of the page size.
 *
 */

#define ROUND_TO_PAGES(Size)  (((ULONG)(Size) + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1))

/*
 *
 * ULONG
 * BYTES_TO_PAGES (
 *     IN ULONG Size
 *     )
 *
 * Routine Description:
 *
 *     The BYTES_TO_PAGES macro takes the size in bytes and calculates the
 *     number of pages required to contain the bytes.
 *
 * Arguments:
 *
 *     Size - Size in bytes.
 *
 * Return Value:
 *
 *     Returns the number of pages required to contain the specified size.
 *
 */

#define BYTES_TO_PAGES(Size)  (((ULONG)(Size) >> PAGE_SHIFT) + \
                               (((ULONG)(Size) & (PAGE_SIZE - 1)) != 0))

/*
 *
 * ULONG
 * BYTE_OFFSET (
 *     IN PVOID Va
 *     )
 *
 * Routine Description:
 *
 *     The BYTE_OFFSET macro takes a virtual address and returns the byte offset
 *     of that address within the page.
 *
 * Arguments:
 *
 *     Va - Virtual address.
 *
 * Return Value:
 *
 *     Returns the byte offset portion of the virtual address.
 *
 */

#define BYTE_OFFSET(Va) ((ULONG)(Va) & (PAGE_SIZE - 1))

/*
 *
 * PVOID
 * PAGE_ALIGN (
 *     IN PVOID Va
 *     )
 *
 * Routine Description:
 *
 *     The PAGE_ALIGN macro takes a virtual address and returns a page-aligned
 *     virtual address for that page.
 *
 * Arguments:
 *
 *     Va - Virtual address.
 *
 * Return Value:
 *
 *     Returns the page aligned virtual address.
 *
 */

#undef PAGE_ALIGN  /* asm/page.h defines this; it's not compatible */

#define PAGE_ALIGN(Va) ((PVOID)((ULONG)(Va) & ~(PAGE_SIZE - 1))) 


/*
 *
 * ULONG
 * ADDRESS_AND_SIZE_TO_SPAN_PAGES (
 *     IN PVOID Va,
 *     IN ULONG Size
 *     )
 *
 * Routine Description:
 *
 *     The ADDRESS_AND_SIZE_TO_SPAN_PAGES macro takes a virtual address and
 *     size and returns the number of pages spanned by the size.
 *
 * Arguments:
 *
 *     Va - Virtual address.
 *
 *     Size - Size in bytes.
 *
 * Return Value:
 *
 *     Returns the number of pages spanned by the size.
 *
 */

#define ADDRESS_AND_SIZE_TO_SPAN_PAGES(Va,Size) \
   ((((ULONG)((ULONG)(Size) - 1L) >> PAGE_SHIFT) + \
   (((((ULONG)(Size-1)&(PAGE_SIZE-1)) + ((ULONG)Va & (PAGE_SIZE -1)))) >> PAGE_SHIFT)) + 1L)

#define COMPUTE_PAGES_SPANNED(Va, Size) \
    ((((ULONG)Va & (PAGE_SIZE -1)) + (Size) + (PAGE_SIZE - 1)) >> PAGE_SHIFT)

/*
 *
 * PVOID
 * MmGetMdlVirtualAddress (
 *     IN PMDL Mdl
 *     )
 *
 * Routine Description:
 *
 *     The MmGetMdlVirtualAddress returns the virual address of the buffer
 *     described by the Mdl.
 *
 * Arguments:
 *
 *     Mdl - Pointer to an MDL.
 *
 * Return Value:
 *
 *     Returns the virtual address of the buffer described by the Mdl
 *
 */

#define MmGetMdlVirtualAddress(Mdl)  ((PVOID) ((PCHAR) (Mdl)->StartVa + (Mdl)->ByteOffset))

/*
 *
 * ULONG
 * MmGetMdlByteCount (
 *     IN PMDL Mdl
 *     )
 *
 * Routine Description:
 *
 *     The MmGetMdlByteCount returns the length in bytes of the buffer
 *     described by the Mdl.
 *
 * Arguments:
 *
 *     Mdl - Pointer to an MDL.
 *
 * Return Value:
 *
 *     Returns the byte count of the buffer described by the Mdl
 *
 */

#define MmGetMdlByteCount(Mdl)  ((Mdl)->ByteCount)

/*
 *
 * ULONG
 * MmGetMdlByteOffset (
 *     IN PMDL Mdl
 *     )
 *
 * Routine Description:
 *
 *     The MmGetMdlByteOffset returns the byte offset within the page
 *     of the buffer described by the Mdl.
 *
 * Arguments:
 *
 *     Mdl - Pointer to an MDL.
 *
 * Return Value:
 *
 *     Returns the byte offset within the page of the buffer described by the Mdl
 *
 */

#define MmGetMdlByteOffset(Mdl)  ((Mdl)->ByteOffset)

/****************************************************************************
 *  Doubly-linked list manipulation routines.  Implemented as macros
 *  but logically these are procedures.
 */

/*
 *  VOID
 *  InitializeListHead(
 *      PLIST_ENTRY ListHead
 *      );
 */

#define InitializeListHead(ListHead) (\
    (ListHead)->Flink = (ListHead)->Blink = (ListHead))

/*
 *  BOOLEAN
 *  IsListEmpty(
 *      PLIST_ENTRY ListHead
 *      );
 */

#define IsListEmpty(ListHead) \
    ((ListHead)->Flink == (ListHead))

/*
 *  PLIST_ENTRY
 *  RemoveHeadList(
 *      PLIST_ENTRY ListHead
 *      );
 */

#define RemoveHeadList(ListHead) \
    (ListHead)->Flink;\
    {RemoveEntryList((ListHead)->Flink)}

/*
 *  PLIST_ENTRY
 *  RemoveTailList(
 *      PLIST_ENTRY ListHead
 *      );
 */

#define RemoveTailList(ListHead) \
    (ListHead)->Blink;\
    {RemoveEntryList((ListHead)->Blink)}

/*
 *  VOID
 *  RemoveEntryList(
 *      PLIST_ENTRY Entry
 *      );
 */

#define RemoveEntryList(Entry) {\
    PLIST_ENTRY _EX_Blink;\
    PLIST_ENTRY _EX_Flink;\
    _EX_Flink = (Entry)->Flink;\
    _EX_Blink = (Entry)->Blink;\
    _EX_Blink->Flink = _EX_Flink;\
    _EX_Flink->Blink = _EX_Blink;\
    }

/*
 *  VOID
 *  InsertTailList(
 *      PLIST_ENTRY ListHead,
 *      PLIST_ENTRY Entry
 *      );
 */

#define InsertTailList(ListHead,Entry) {\
    PLIST_ENTRY _EX_Blink;\
    PLIST_ENTRY _EX_ListHead;\
    _EX_ListHead = (ListHead);\
    _EX_Blink = _EX_ListHead->Blink;\
    (Entry)->Flink = _EX_ListHead;\
    (Entry)->Blink = _EX_Blink;\
    _EX_Blink->Flink = (Entry);\
    _EX_ListHead->Blink = (Entry);\
    }

/*
 *  VOID
 *  InsertHeadList(
 *      PLIST_ENTRY ListHead,
 *      PLIST_ENTRY Entry
 *      );
 */

#define InsertHeadList(ListHead,Entry) {\
    PLIST_ENTRY _EX_Flink;\
    PLIST_ENTRY _EX_ListHead;\
    _EX_ListHead = (ListHead);\
    _EX_Flink = _EX_ListHead->Flink;\
    (Entry)->Flink = _EX_Flink;\
    (Entry)->Blink = _EX_ListHead;\
    _EX_Flink->Blink = (Entry);\
    _EX_ListHead->Flink = (Entry);\
    }

typedef unsigned int vm_offset_t;

#define trunc_page(x)            ((unsigned)(x) & ~PAGE_MASK)

/* Stuff to make this driver compile on both 2.0 and 2.2. The driver will be
   using the 2.2.x functions, and this code will thunk things for 2.0.x. */

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(a,b,c) ((((a) << 16) + ((b) << 8) + (c))
#endif /* KERNEL_VERSION */


#if (LINUX_VERSION_CODE < 0x020100)

#define copy_to_user memcpy_tofs
#define copy_from_user memcpy_fromfs
#define ioremap vremap

#define __initdata
#define __initfunc(func) func
#else 
#include <asm/uaccess.h>
#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0) */

#endif

