/**********************************************************************

    Module: kmi_private.h

        Product: kernel mode driver for "PowerCrypt" encryption
        acceleration board based on Hi/fn 7751 chip
        by Ashley Laurent, Inc.

    ---------------------------------------------------------------

    Copyright:

        Ashley Laurent, Inc., 1999
        All Rights Reserved.

    ---------------------------------------------------------------

    Functional Description:

        Routines that KM interface module exposes to the driver's internal
        components.

    ---------------------------------------------------------------

    Environment:

        NT or BSD kernel mode

    ---------------------------------------------------------------

    Author:

        Stan Shkolnyy

    ---------------------------------------------------------------

    Revision history:

        *   15-Feb-99   initial revision

**********************************************************************/

#ifndef __KMI_PRIVATE_H__
#define __KMI_PRIVATE_H__

/****************************************************************************
 * Ensure inclusion of necessary headers
 */

/*---------------------------------------------------------------------------
 * pwrcryptioctl.h must be included before this file since it contains defines
 * for entry point parameters - PIOCTL_PWRCRYPT_XXX_INPUT_OUTPUT
 */
#ifndef __PWRCRYPTIOCTL_H__
    #error include 'pwrcryptioctl.h' before including this file
#endif

/****************************************************************************
    The following two funcs are used by an O/S-dependent code to switch this
    KM interface to enabled or disabled state. This state is verified by all
    external interface functions (defined above) before they decide to proceed.
 
    Activate() is called once the O/S-dep. code has constructed all the
    O/S-indep. objects and the driver become ready to be entered by clients.
 
    Deactivate() is called when the driver is unloaded.
 
    This mechanism works exactly like opening/shutting a door through which
    clients enter. This means that threads may be inside of the driver at the
    moment when the KM interface is being disabled. This obstacle must be taken
    into account in the shutdown procedure (BUGBUG).
****************************************************************************/
void
KmInterfaceEnable
    ( void
    );

void
KmInterfaceDisable
    ( void
    );

/****************************************************************************
    The call to the KmInterfaceCompleteRequest indicates that the caller has
    completed all processing for a given request and is returning the given Calcreq to the KM interface.
    KM interface performs necessary cleanup and notifies the client.
    This service is PwrCrypt internal and is called by a PROCESSOR
****************************************************************************/
void
KmInterfaceCompleteCalcRequest
    (
        PIOCTL_PWRCRYPT_CALCULATE_INPUT_OUTPUT  pCalcReq
    );

#endif
