#include <ntddk.h>
#include "debug.h"
#include "memtrack.h"

/****************************************************************************

NTSTATUS
MyCloneUnicodeString(
    IN OUT PUNICODE_STRING  pusDstPath,
    IN     PUNICODE_STRING  pusSrcPath,
    IN     USHORT           nSizeDelta
    )

Description:

    Allocates memory for dst unicode str. buffer and copies src string

Arguments:

    pusDstPath - ptr to a non-initialized unicode string structure.
    pusSrcPath - ptr to a initialized unicode string structure.
    nSizeDelta - the allocated dst string buffer is longer by this value

Return Value:
    NTSTATUS

****************************************************************************/

NTSTATUS
MyCloneUnicodeString(
    IN OUT PUNICODE_STRING  pusDstPath,
    IN     PUNICODE_STRING  pusSrcPath,
    IN     USHORT           nSizeDelta
    )
{
    NTSTATUS status;

    status = STATUS_SUCCESS;

    RtlInitUnicodeString( pusDstPath, NULL);

    pusDstPath->MaximumLength = pusSrcPath->MaximumLength + nSizeDelta;

    pusDstPath->Buffer = MemAllocatePool( NonPagedPool,
                                          pusDstPath->MaximumLength
                                         );
    if ( pusDstPath->Buffer != NULL)
    {
        RtlCopyUnicodeString( pusDstPath, pusSrcPath);
    }
    else
    {
        DebugPrint((1, "Unable to clone unicode string\n"));
        status = STATUS_INSUFFICIENT_RESOURCES;
    }
    return status;
}

/****************************************************************************
Replaces the last character in the passed UNICODE_STRING with character
representation of the passed number. ("ProcessorX", 5) makes "Processor5"
****************************************************************************/
void ReplaceOrderNum(
    PUNICODE_STRING pusStringX,
    USHORT nOrderNum
    )
{
//    ASSERT( pusStringX->Buffer[ pusStringX->Length / (USHORT) sizeof( USHORT) - 1] == L'X');
    ASSERT( nOrderNum >= 0 && nOrderNum <= 9);
    pusStringX->Buffer[ pusStringX->Length / (USHORT) sizeof( USHORT) - 1] =
            (USHORT)'0' + (USHORT)nOrderNum;
}
