/**********************************************************************

    Module: sg_desc.h

        Product: kernel mode driver for "PowerCrypt" encryption
        acceleration board based on Hi/fn 7751 chip
        by Ashley Laurent, Inc.

    ---------------------------------------------------------------

    Copyright:

        Ashley Laurent, Inc., 1999
        All Rights Reserved.

    ---------------------------------------------------------------

    Functional Description:

        Layout and accessors of a single scatter/gather memory descriptor
        used by PROCESSOR and the 7751 DMA unit.

    ---------------------------------------------------------------

    Environment:

        NT or BSD kernel mode

    ---------------------------------------------------------------

    Author:

        Stan Shkolnyy

    ---------------------------------------------------------------

    Revision history:

        *   15-Feb-99   initial revision

**********************************************************************/

#ifndef __SG_DESC_H__
#define __SG_DESC_H__

#ifdef _WIN32
    #pragma pack(push, _SGDESC_STRUCT_DEF_, 1)
#endif

#define SG_DESC_MAX_CNT        0xfffc

/****************************************************************************
 * Scatter/gather descriptor - the DMA engines are driven by such structures
 * placed solidly ( as arrays) in the PC RAM.
 */
typedef struct _SG_DESC {
    USHORT  DataSize;
    USHORT  CtlFlags;
    ULONG   DataPhysAddr;
} SG_DESC, *PSG_DESC;

#define SG_DESC_FLAG_VALID     0x8000
#define SG_DESC_FLAG_JUMP      0x4000
#define SG_DESC_FLAG_LAST      0x2000
#define SG_DESC_FLAG_MASKDONE  0x0200

/**********************************************************************
 * PUBLIC methods
 */
#define SgDescSetFlags(  pThis, flags)   \
    ( (pThis)->CtlFlags = flags)

#define SgDescAddFlags(  pThis, flags)   \
    ( (pThis)->CtlFlags |= flags)

#define SgDescGetFlags(  pThis)   \
    ( (pThis)->CtlFlags)

#define SgDescSetDataPhysAddr(  pThis, addr) \
    ( (pThis)->DataPhysAddr = addr)

#define SgDescGetDataPhysAddr(  pThis) \
    ( (pThis)->DataPhysAddr)

#define SgDescSetDataSize(  pThis, size) \
    ( (pThis)->DataSize = size)

#define SgDescGetDataSize(  pThis) \
    ( (pThis)->DataSize)

/**********************************************************************
 * PRIVATE methods
 */

#ifdef _WIN32
    #pragma pack(pop, _SGDESC_STRUCT_DEF_)
#endif

#endif
