#include "osindependence.h"
#include "debug.h"
#include "watch.h"

/****************************************************************************
****************************************************************************/
void
WatchConstructor(
    IN OUT  PWATCH pThis
    )
{
    int i;

    DebugPrint((2, "WatchConstructor\n"));
    pThis->Type = WATCH_SIGN;

    pThis->pThis = pThis;

    for( i = 0; i < WATCH_ENTRIES; i++)
    {
        pThis->md[ i].pMem = NULL;
        pThis->md[ i].nSize = 0;
        pThis->md[ i].nCS = 0;
        pThis->md[ i].Pad = 0xeeeeeeee;
    }
    pThis->bRefresh = TRUE;
}

/****************************************************************************
****************************************************************************/
void
WatchInit(
    IN OUT  PWATCH  pThis
    )
{
    DebugPrint((2, "WatchInit\n"));
    ASSERT( pThis->Type == WATCH_SIGN);

    // ntoskrnl
    pThis->md[ 1].pMem = (PUCHAR) 0x80100380;
    pThis->md[ 1].nSize = 0x41f97;

    // hal
    pThis->md[ 2].pMem = (PUCHAR) 0x800102e0;
    pThis->md[ 2].nSize = 0x4f84;

    // fastfat
    pThis->md[ 3].pMem = (PUCHAR) 0x801df2a0;
    pThis->md[ 3].nSize = 0x1a3d;

    // win32k
    pThis->md[ 4].pMem = (PUCHAR) 0xa00002a0;
    pThis->md[ 4].nSize = 0x11bd6e;

    ASSERT(0);
    WatchRecalc( pThis);
}

/****************************************************************************
****************************************************************************/
void
WatchRecalc(
    IN OUT  PWATCH  pThis
    )
{
    int i;

    DebugPrint((2, "    WtRecalc\n"));
    ASSERT( pThis->Type == WATCH_SIGN);

    for( i = 0; i < WATCH_ENTRIES; i++)
    {
        if ( pThis->md[ i].pMem != NULL && pThis->md[ i].nSize != 0)
        {
            PUCHAR  pCurByte;
            ULONG   Cnt;

            pCurByte = pThis->md[ i].pMem;
            Cnt = pThis->md[ i].nSize;

            pThis->md[ i].nCS = 0;

            while ( Cnt != 0)
            {
                pThis->md[ i].nCS += *pCurByte;
                pCurByte++;
                Cnt--;
            }
        }
    }
    pThis->bRefresh = FALSE;
}

/****************************************************************************
****************************************************************************/
void
WatchDestructor(
    IN PWATCH pThis
    )
{
    DebugPrint((2, "WatchDestructor\n"));
    ASSERT( pThis->Type == WATCH_SIGN);
}

/****************************************************************************
****************************************************************************/
void
WatchCheck(
    IN PWATCH pThis
    )
{
    int i;

    DebugPrint((2, "WatchCheck\n"));
    ASSERT( pThis->Type == WATCH_SIGN);

    for( i = 0; i < WATCH_ENTRIES; i++)
    {
        if ( pThis->md[ i].pMem != NULL && pThis->md[ i].nSize != 0)
        {
            PUCHAR  pCurByte;
            ULONG   Cnt;
            ULONG   nChkSum;

            pCurByte = pThis->md[ i].pMem;
            Cnt = pThis->md[ i].nSize;
            nChkSum = 0;
            while ( Cnt != 0)
            {
                nChkSum += *pCurByte;
                pCurByte++;
                Cnt--;
            }
            if ( nChkSum != pThis->md[ i].nCS)
            {
                DebugPrint((2, "    checksum error in entry %d\n", i));
                ASSERT(0);
                pThis->bRefresh = TRUE;
            }
        }
    }

    if ( pThis->bRefresh)
    {
        WatchRecalc( pThis);
    }
}
