/**********************************************************************

    Module: xfer.h

        Product: kernel mode driver for "PowerCrypt" encryption
        acceleration board based on Hi/fn 7751 chip
        by Ashley Laurent, Inc.

    ---------------------------------------------------------------

    Copyright:

        Ashley Laurent, Inc., 1999
        All Rights Reserved.

    ---------------------------------------------------------------

    Functional Description:

        Defines constants that control size of DMA transfers and
        overlapping the transfers in time. Used by PROCESSOR.

    ---------------------------------------------------------------

    Environment:

        NT or BSD kernel mode

    ---------------------------------------------------------------

    Author:

        Stan Shkolnyy

    ---------------------------------------------------------------

    Revision history:

        *   15-Feb-99   initial revision

**********************************************************************/

#ifndef __XFER_H__
#define __XFER_H__

#include "instrset.h"   // for PROCESSOR_MAX_XFER_SIZE

/*
 * This many 7751 "instructions" can be simultaneously in progress on the processor
 */
#define PROCESSOR_QUEUE_LENGTH      2

/*
 * A data ring must be big enough to contain the descriptors for two biggest
 * transfers + 1 "jump" descriptor
 */
#define DATA_RING_LEN           2 * ((PROCESSOR_MAX_XFER_SIZE / PAGE_SIZE) + 2) + 1

/*
 * Command and result rings each contain 1 descriptor per simultaneously active
 * instruction + each 1 "jump" descriptors at the tail of the ring
 */
#define COM_AND_RES_RING_LEN   PROCESSOR_QUEUE_LENGTH + 1

#endif
