/*++

Copyright (c) 1997-1998  Microsoft Corporation

Module Name:

    debug.c

Abstract:

    General debug print routine.  

Author:

    Steve Dziok (SteveDz)

Environment:

    Kernel mode

Revision History:


--*/

#include <stdarg.h>
#include <stdio.h>
#include <ntddk.h>
#include "debug.h"
#include "driverdef.h"

#if DBG

VOID
DebugPrintRoutine(
    ULONG PrintLevel,
    PCCHAR DebugMessage,
    ...
    )
/*++

Routine Description:

    Debug print routine.

Arguments:

    Debug print level between 0 and 3, with 3 being the most verbose.

Return Value:

    None.

--*/
{
    va_list ap;

    va_start(ap, DebugMessage);

    if (PrintLevel <= DebugPrintLevel) {

        (VOID) vsprintf(DebugBuffer, DebugMessage, ap);

#ifdef DRIVER_NAME
  
        DbgPrint(DRIVER_NAME ": ");
        
#else

#pragma message(" ")
#pragma message("*********************************************************************** ")
#pragma message("*                                                                     * ")
#pragma message("*  DEBUG.C:                                                           * ")
#pragma message("*                                                                     * ")
#pragma message("*  Define DRIVER_NAME in DRIVER.H to customize the DbgPrint routine.  * ")
#pragma message("*                                                                     * ")
#pragma message("*********************************************************************** ")
#pragma message(" ")
        
#endif

        DbgPrint(DebugBuffer);
    }

    va_end(ap);

}

#else

VOID
DebugPrintRoutine(
    ULONG PrintLevel,
    PCCHAR DebugMessage,
    ...
    )
{

}

#endif
