/**********************************************************************

    Module: osindependence.c

        Product: kernel mode driver for "PowerCrypt" encryption
        acceleration board based on Hi/fn 7751 chip
        by Ashley Laurent, Inc.

    ---------------------------------------------------------------

    Copyright:

        Ashley Laurent, Inc., 1999
        All Rights Reserved.

    ---------------------------------------------------------------

    Functional Description:

        O/S-independent wrappers for O/S dependent API functions that
        O/S-independent code needs.

    ---------------------------------------------------------------

    Environment:

        NT or BSD kernel mode

    ---------------------------------------------------------------

    Author:

        Stan Shkolnyy

    ---------------------------------------------------------------

    Revision history:

        *   7-Mar-99   initial revision

**********************************************************************/

#include "osindependence.h"

/****************************************************************************
 * Compile different function bodies with the same names depending on O/S
 */

/******************
 * NT
 */
#ifdef _NT_DRIVER_BUILD_

/*---------------------------------------------------------------------------
 * Doubly-linked list - unsafe add/remove operations, needs protection
 */
void
PCrptListInit
    (
        IN OUT  PPCRPT_LIST pThis
    )
{
    InitializeListHead( &pThis->ListHead);
}

BOOLEAN
PCrptIsListEmpty
    (
        IN OUT  PPCRPT_LIST pThis
    )
{
    BOOLEAN bEmpty;
    bEmpty = IsListEmpty( &pThis->ListHead);
    return bEmpty;
}

void
PCrptListInsertTail
    (
        IN OUT  PPCRPT_LIST pThis,
        PPCRPT_LIST_ENTRY   ListEntry
    )
{
    InsertTailList( &pThis->ListHead, ListEntry);
}

void
PCrptListInsertHead(
    IN OUT  PPCRPT_LIST pThis,
    PPCRPT_LIST_ENTRY   ListEntry
    )
{
    InsertHeadList( &pThis->ListHead, ListEntry);
}

PPCRPT_LIST_ENTRY
PCrptListRemoveHead(
    IN OUT  PPCRPT_LIST pThis
    )
{
    PPCRPT_LIST_ENTRY pEntry;
    pEntry = ( PPCRPT_LIST_ENTRY) RemoveHeadList( &pThis->ListHead);
    return pEntry;
}

#endif /* _NT_DRIVER_BUILD_ */

/******************
 * BSD
 */
#ifdef  _BSD_DRIVER_BUILD_

/*---------------------------------------------------------------------------
 * Doubly-linked list - unsafe add/remove operations, needs protection
 */
void
PCrptListInit
    (
        IN OUT  PPCRPT_LIST pThis
    )
{
    InitializeListHead( &pThis->ListHead);
}

BOOLEAN
PCrptIsListEmpty
    (
        IN OUT  PPCRPT_LIST pThis
    )
{
    BOOLEAN bEmpty;
    bEmpty = IsListEmpty( &pThis->ListHead);
    return bEmpty;
}

void
PCrptListInsertTail
    (
        IN OUT  PPCRPT_LIST pThis,
        PPCRPT_LIST_ENTRY   ListEntry
    )
{
    InsertTailList( &pThis->ListHead, ListEntry);
}

void
PCrptListInsertHead(
    IN OUT  PPCRPT_LIST pThis,
    PPCRPT_LIST_ENTRY   ListEntry
    )
{
    InsertHeadList( &pThis->ListHead, ListEntry);
}

PPCRPT_LIST_ENTRY
PCrptListRemoveHead(
    IN OUT  PPCRPT_LIST pThis
    )
{
    PPCRPT_LIST_ENTRY pEntry;
    pEntry = ( PPCRPT_LIST_ENTRY) RemoveHeadList( &pThis->ListHead);
    return pEntry;
}

#endif /* _BSD_DRIVER_BUILD_ */
