/**********************************************************************

    Module: symlink.h

        Product: kernel mode driver providing user mode interface
        to the PwrCrypt driver for encryption acceleration board
        based on Hi/fn 7751 chip by Ashley Laurent, Inc.

    ---------------------------------------------------------------

    Copyright:

        Ashley Laurent, Inc., 1999
        All Rights Reserved.

    ---------------------------------------------------------------

    Functional Description:

        Defines structure and methods of the SymLink object that
        represents the symbolic link created by the driver under DosDevices.

    ---------------------------------------------------------------

    Environment:

        NT or BSD kernel and user mode

    ---------------------------------------------------------------

    Author:

        Stan Shkolnyy

    ---------------------------------------------------------------

    Revision history:

        *   16-Mar-99   initial revision

**********************************************************************/

typedef struct _SYMLINK
{
    ULONG           Type;

    /*
     * If this string is not empty, it is the name of an _existent_
     * symbolic link under \DosDevices
     */
    UNICODE_STRING      usLinkName;

} SYMLINK, *PSYMLINK;

void
SymlinkConstructor
    (
        IN PSYMLINK pThis
    );

NTSTATUS
SymlinkCreateSymbolicLink
    (
        IN PSYMLINK pThis,
        IN UCHAR *szLinkName,
        IN PUNICODE_STRING pusDeviceName
    );

void
SymlinkDeleteSymbolicLink
    (
        IN PSYMLINK pThis
    );

void
SymlinkDestructor
    (
        IN PSYMLINK pThis
    );
