#ifndef __DRIVERDEF_H__
#define __DRIVERDEF_H__

/****************************************************************************
 * Miscelanious global type constants
 */

/*
 * PowerCrypt 7751 board VendorId and DeviceId
 */
#define PWRCRYPT_PCI_VENDOR_ID  0x13A3
#define PWRCRYPT_PCI_DEVICE_ID  0x0005

/*
 * The driver debug name is used by DebugPrint() routine to prepend the output
 */
#define DRIVER_DEBUG_NAME     "PC"

/*
 * Maximum number of devices this driver supports.
 * Scales the driver's compiled-in internal tables.
 */
#define MAXIMUM_DEVICES_SUPPORTED   4

/*
 * Unlock key used by the driver (in ProcUnlock7751()) to switch the 7751
 * to higher security levels. The values in the key must correspond to contents
 * of the board's EEPROM.
 */

#define arraysize(a)    sizeof(a) * sizeof( a[0])

typedef struct _UNLOCK_KEY
{
    /* To unlock exportable encryption and authentication, these bytes
     * must match the first nine bytes of Unlock Key in the EEPROM.
     *
     * The Reference Hardware has this key set to all zeroes, but a
     * secure implementation should have random values.
     */
    unsigned char   Level1Value     [ 9];

    /* To unlock strong encryption, these bytes must matcch the last
     * four bytes of Unlock Key in the EEPROM.                      
     *
     * The Reference Hardware has this key set to all zeroes, but a
     * secure implementation should have random values.            
     *
     * If exportable encryption is desirable, these values should *not*
     * match the last four bytes of the Unlock Key in the EEPROM so
     * that the remainder of the unlock key cannot be extracted from
     * the code or used by simply changing desiredState from 1 to 2.
     */
    unsigned char   Level2Appendix  [ 4];

} UNLOCK_KEY, *PUNLOCK_KEY;

/*
 * 7751 can operate on 3 different security levels which differ by the set
 * of functions implemented.
 */
typedef enum
{
    INITIAL_LOCKED = 0,	/* default after power on */
    PARTIALLY_UNLOCKED,
    FULLY_UNLOCKED

} SecurLevel;


/*
 * Tags for marking objects. Used in methods to check passed "pThis"es
 */
#define PROCESSOR_SIGN      0x01010101
#define PROCBASE_SIGN       0x02020202
#define SLOTMAN_SIGN        0x03030303
#define CONFIG_SIGN         0x04040404
#define DISPATCHER_SIGN     0x05050505
#define SG_CTL_AREA_SIGN    0x06060606
#define SG_RING_SIGN        0x07070707
#define DMA_CTX_SIGN        0x08080808
#define SLOTRACK_SIGN       0x09090909


#endif /* __DRIVERDEF_H__ */
